'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getButtons = getButtons;

var _simplemde = require('simplemde');

var _simplemde2 = _interopRequireDefault(_simplemde);

var _i18n = require('./i18n');

var _i18n2 = _interopRequireDefault(_i18n);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buttons = {
  // built-in buttons
  '|': '|',
  'bold': {
    name: 'bold',
    action: _simplemde2.default.toggleBold,
    className: 'fa fa-bold'
  },
  'italic': {
    name: 'italic',
    action: _simplemde2.default.toggleItalic,
    className: 'fa fa-italic'
  },
  'strikethrough': {
    name: 'strikethrough',
    action: _simplemde2.default.toggleStrikethrough,
    className: 'fa fa-strikethrough'
  },
  'heading': {
    name: 'heading',
    action: _simplemde2.default.toggleHeadingSmaller,
    className: 'fa fa-header'
  },
  'heading-smaller': {
    name: 'heading-smaller',
    action: _simplemde2.default.toggleHeadingSmaller,
    className: 'fa fa-header fa-header-x fa-header-smaller'
  },
  'heading-bigger': {
    name: 'heading-bigger',
    action: _simplemde2.default.toggleHeadingBigger,
    className: 'fa fa-header fa-header-x fa-header-bigger'
  },
  'separator-1': 'separator-1',
  'heading-1': {
    name: 'heading-1',
    action: _simplemde2.default.toggleHeading1,
    className: 'fa fa-header fa-header-x fa-header-1'
  },
  'heading-2': {
    name: 'heading-2',
    action: _simplemde2.default.toggleHeading2,
    className: 'fa fa-header fa-header-x fa-header-2'
  },
  'heading-3': {
    name: 'heading-3',
    action: _simplemde2.default.toggleHeading3,
    className: 'fa fa-header fa-header-x fa-header-3'
  },
  'code': {
    name: 'code',
    action: _simplemde2.default.toggleCodeBlock,
    className: 'fa fa-code'
  },
  'quote': {
    name: 'quote',
    action: _simplemde2.default.toggleBlockquote,
    className: 'fa fa-quote-left'
  },
  'unordered-list': {
    name: 'unordered-list',
    action: _simplemde2.default.toggleUnorderedList,
    className: 'fa fa-list-ul'
  },
  'ordered-list': {
    name: 'ordered-list',
    action: _simplemde2.default.toggleOrderedList,
    className: 'fa fa-list-ol'
  },
  'clean-block': {
    name: 'clean-block',
    action: _simplemde2.default.cleanBlock,
    className: 'fa fa-eraser fa-clean-block'
  },
  'separator-2': 'separator-2',
  'link': {
    name: 'link',
    action: _simplemde2.default.drawLink,
    className: 'fa fa-link'
  },
  'image': {
    name: 'image',
    action: _simplemde2.default.drawImage,
    className: 'fa fa-picture-o'
  },
  'table': {
    name: 'table',
    action: _simplemde2.default.drawTable,
    className: 'fa fa-table'
  },
  'horizontal-rule': {
    name: 'horizontal-rule',
    action: _simplemde2.default.drawHorizontalRule,
    className: 'fa fa-minus'
  },
  'separator-3': 'separator-3',
  'preview': {
    name: 'preview',
    action: _simplemde2.default.togglePreview,
    className: 'fa fa-eye no-disable'
  },
  'side-by-side': {
    name: 'side-by-side',
    action: _simplemde2.default.toggleSideBySide,
    className: 'fa fa-columns no-disable no-mobile'
  },
  'fullscreen': {
    name: 'fullscreen',
    action: _simplemde2.default.toggleFullScreen,
    className: 'fa fa-arrows-alt no-disable no-mobile'
  },
  'separator-4': 'separator-4',
  'guide': {
    name: 'guide',
    className: 'fa fa-question-circle'
  },
  'separator-5': 'separator-5',
  'undo': {
    name: 'undo',
    action: _simplemde2.default.undo,
    className: 'fa fa-undo no-disable'
  },
  'redo': {
    name: 'redo',
    action: _simplemde2.default.redo,
    className: 'fa fa-repeat no-disable'
  },
  // custom buttons
  'true-false-question': {
    name: 'true-false-question',
    action: drawTrueFalseQuestion,
    className: 'fa fa-question'
  },
  'multiple-choice-question': {
    name: 'multiple-choice-question',
    action: toggleMultiChoiceQuestion,
    className: 'fa fa-check-square-o'
  },
  'blank-filling-question': {
    name: 'blank-filling-question',
    action: drawBlankFillingQuestion,
    className: 'fa fa-pencil-square-o'
  },
  'latex': {
    name: 'latex',
    action: drawLatex,
    className: 'fa fa-superscript'
  },
  'upload-image': {
    name: 'upload-image',
    className: 'fa fa-picture-o'
  },
  'upload-file': {
    name: 'upload-file',
    className: 'fa fa-file-text-o'
  }
};

function drawTrueFalseQuestion(editor) {
  var cm = editor.codemirror;
  var selectedText = cm.getSelection();
  cm.replaceSelection(selectedText + '~@[]()');
  cm.focus();
}

function toggleMultiChoiceQuestion(editor) {
  var cm = editor.codemirror;
  var startPoint = cm.getCursor('start');
  var endPoint = cm.getCursor('end');
  var prefixReg = /^[A-Z]\. */;

  var lines = [];
  for (var i = startPoint.line; i <= endPoint.line; i++) {
    lines.push(cm.getLine(i));
  }
  var isMultiChoiceQuestion = lines.reduce(function (prev, line) {
    return prev && prefixReg.test(line);
  }, true);

  lines.forEach(function (line, index) {
    var result = line;
    if (isMultiChoiceQuestion) {
      result = line.replace(prefixReg, '');
    } else {
      result = 'A. ' + line.replace(prefixReg, '');
    }
    cm.replaceRange(result, {
      line: startPoint.line + index,
      ch: 0
    }, {
      line: startPoint.line + index,
      ch: 99999999999999
    });
  });

  cm.focus();
}

function drawBlankFillingQuestion(editor) {
  var cm = editor.codemirror;
  var selectedText = cm.getSelection();
  var startPoint = cm.getCursor('start');
  var endPoint = cm.getCursor('end');
  cm.replaceSelection('@@[' + selectedText + ']()');
  cm.focus();

  if (startPoint.line !== endPoint.line || startPoint.ch !== endPoint.ch) {
    endPoint.ch += 3;
  }
  startPoint.ch += 3;
  cm.setSelection(startPoint, endPoint);
}

function drawLatex(editor) {
  var cm = editor.codemirror;
  var selectedText = cm.getSelection();
  var startPoint = cm.getCursor('start');
  var endPoint = cm.getCursor('end');
  cm.replaceSelection('$$' + selectedText + '$$');
  cm.focus();

  if (startPoint.line !== endPoint.line || startPoint.ch !== endPoint.ch) {
    endPoint.ch += 2;
  }
  startPoint.ch += 2;
  cm.setSelection(startPoint, endPoint);
}

function uploadAndInsertFile(uploader, accepts, insertAsImage) {
  if (typeof uploader !== 'function') return _simplemde2.default.drawImage;

  return function (editor) {
    var fileInput = document.createElement('input');
    fileInput.type = 'file';
    fileInput.accept = accepts.join(', ');
    fileInput.setAttribute('multiple', 'multiple');

    fileInput.onchange = function (e) {
      var files = e.target.files;
      if (e.target.files.length <= 0) return;
      Promise.all(Array.from(files).map(function (file) {
        return uploader(file);
      })).then(function (uploadedFiles) {
        var cm = editor.codemirror;
        var startPoint = cm.getCursor('start');
        var uploadedFileStr = uploadedFiles.map(function (file, index) {
          return insertAsImage ? '\n![' + (files[index].name || '') + '](' + file.url + ')' : '\n[' + (files[index].name || '') + '](' + file.url + ')';
        }).join('') + '\n';
        cm.replaceSelection(uploadedFileStr);
        cm.focus();

        startPoint.ch = 0;
        startPoint.line += uploadedFiles.length + 1;
        cm.setCursor(startPoint);
      });
    };
    fileInput.click();
  };
}

function getButtons(buttonNames) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var lng = config.lng,
      uploader = config.uploader;

  if (!buttonNames) return;
  var localButtons = {};
  Object.keys(buttons).forEach(function (key) {
    var button = buttons[key];
    if (typeof button === 'string') {
      localButtons[key] = button;
    } else {
      localButtons[key] = Object.assign({}, button, {
        title: _i18n2.default.t('button:' + key, { lng: lng })
      });
      if (key === 'upload-image') {
        localButtons[key].action = uploadAndInsertFile(uploader, ['image/*'], true);
      }
      if (key === 'upload-file') {
        localButtons[key].action = uploadAndInsertFile(uploader, [
        // https://github.com/pintia/sparkling-daydream/blob/master/src/main/kotlin/cn/patest/sparklingdaydream/util/NosUtil.kt
        'application/pdf', 'application/msword', // .doc
        'application/vnd.ms-word', // .doc
        'application/vnd.openxmlformats-officedocument.wordprocessingml.document', // .docx
        'application/vnd.ms-powerpoint', // .ppt
        'application/vnd.openxmlformats-officedocument.presentationml.presentation', // .pptx
        'application/vnd.ms-excel', // .xls
        'application/vnd.ms-spreadsheet', // .xls
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'] // .xlsx
        );
      }
      if (key === 'guide' && config.guide) {
        localButtons[key].action = config.guide;
      }
    }
  });
  return buttonNames.map(function (buttonName) {
    return localButtons[buttonName] || buttonName;
  });
}