import SimpleMDE from 'simplemde'

import i18n from './i18n'

const buttons = {
  // built-in buttons
  '|': '|',
  'bold': {
    name: 'bold',
    action: SimpleMDE.toggleBold,
    className: 'fa fa-bold',
  },
  'italic': {
    name: 'italic',
    action: SimpleMDE.toggleItalic,
    className: 'fa fa-italic',
  },
  'strikethrough': {
    name: 'strikethrough',
    action: SimpleMDE.toggleStrikethrough,
    className: 'fa fa-strikethrough',
  },
  'heading': {
    name: 'heading',
    action: SimpleMDE.toggleHeadingSmaller,
    className: 'fa fa-header',
  },
  'heading-smaller': {
    name: 'heading-smaller',
    action: SimpleMDE.toggleHeadingSmaller,
    className: 'fa fa-header fa-header-x fa-header-smaller',
  },
  'heading-bigger': {
    name: 'heading-bigger',
    action: SimpleMDE.toggleHeadingBigger,
    className: 'fa fa-header fa-header-x fa-header-bigger',
  },
  'separator-1': 'separator-1',
  'heading-1': {
    name: 'heading-1',
    action: SimpleMDE.toggleHeading1,
    className: 'fa fa-header fa-header-x fa-header-1',
  },
  'heading-2': {
    name: 'heading-2',
    action: SimpleMDE.toggleHeading2,
    className: 'fa fa-header fa-header-x fa-header-2',
  },
  'heading-3': {
    name: 'heading-3',
    action: SimpleMDE.toggleHeading3,
    className: 'fa fa-header fa-header-x fa-header-3',
  },
  'code': {
    name: 'code',
    action: SimpleMDE.toggleCodeBlock,
    className: 'fa fa-code',
  },
  'quote': {
    name: 'quote',
    action: SimpleMDE.toggleBlockquote,
    className: 'fa fa-quote-left',
  },
  'unordered-list': {
    name: 'unordered-list',
    action: SimpleMDE.toggleUnorderedList,
    className: 'fa fa-list-ul',
  },
  'ordered-list': {
    name: 'ordered-list',
    action: SimpleMDE.toggleOrderedList,
    className: 'fa fa-list-ol',
  },
  'clean-block': {
    name: 'clean-block',
    action: SimpleMDE.cleanBlock,
    className: 'fa fa-eraser fa-clean-block',
  },
  'separator-2': 'separator-2',
  'link': {
    name: 'link',
    action: SimpleMDE.drawLink,
    className: 'fa fa-link',
  },
  'image': {
    name: 'image',
    action: SimpleMDE.drawImage,
    className: 'fa fa-picture-o',
  },
  'table': {
    name: 'table',
    action: SimpleMDE.drawTable,
    className: 'fa fa-table',
  },
  'horizontal-rule': {
    name: 'horizontal-rule',
    action: SimpleMDE.drawHorizontalRule,
    className: 'fa fa-minus',
  },
  'separator-3': 'separator-3',
  'preview': {
    name: 'preview',
    action: SimpleMDE.togglePreview,
    className: 'fa fa-eye no-disable',
  },
  'side-by-side': {
    name: 'side-by-side',
    action: SimpleMDE.toggleSideBySide,
    className: 'fa fa-columns no-disable no-mobile',
  },
  'fullscreen': {
    name: 'fullscreen',
    action: SimpleMDE.toggleFullScreen,
    className: 'fa fa-arrows-alt no-disable no-mobile',
  },
  'separator-4': 'separator-4',
  'guide': {
    name: 'guide',
    className: 'fa fa-question-circle',
  },
  'separator-5': 'separator-5',
  'undo': {
    name: 'undo',
    action: SimpleMDE.undo,
    className: 'fa fa-undo no-disable',
  },
  'redo': {
    name: 'redo',
    action: SimpleMDE.redo,
    className: 'fa fa-repeat no-disable',
  },
  // custom buttons
  'true-false-question': {
    name: 'true-false-question',
    action: drawTrueFalseQuestion,
    className: 'fa fa-question',
  },
  'multiple-choice-question': {
    name: 'multiple-choice-question',
    action: toggleMultiChoiceQuestion,
    className: 'fa fa-check-square-o',
  },
  'blank-filling-question': {
    name: 'blank-filling-question',
    action: drawBlankFillingQuestion,
    className: 'fa fa-pencil-square-o',
  },
  'latex': {
    name: 'latex',
    action: drawLatex,
    className: 'fa fa-superscript',
  },
  'upload-image': {
    name: 'upload-image',
    className: 'fa fa-picture-o',
  },
  'upload-file': {
    name: 'upload-file',
    className: 'fa fa-file-text-o',
  }
}

function drawTrueFalseQuestion(editor) {
  const cm = editor.codemirror
  const selectedText = cm.getSelection()
  cm.replaceSelection(`${selectedText}~@[]()`)
  cm.focus()
}

function toggleMultiChoiceQuestion(editor) {
  const cm = editor.codemirror
  const startPoint = cm.getCursor('start')
  const endPoint = cm.getCursor('end')
  const prefixReg = /^[A-Z]\. */

  const lines = []
  for (let i = startPoint.line; i <= endPoint.line; i++) {
    lines.push(cm.getLine(i))
  }
  const isMultiChoiceQuestion = lines.reduce((prev, line) => prev && (prefixReg.test(line)), true)

  lines.forEach((line, index) => {
    let result = line
    if (isMultiChoiceQuestion) {
      result = line.replace(prefixReg, '')
    } else {
      result = `A. ${line.replace(prefixReg, '')}`
    }
    cm.replaceRange(result, {
      line: startPoint.line + index,
      ch: 0,
    }, {
      line: startPoint.line + index,
      ch: 99999999999999,
    })
  })

  cm.focus()
}

function drawBlankFillingQuestion(editor) {
  const cm = editor.codemirror
  const selectedText = cm.getSelection()
  const startPoint = cm.getCursor('start')
  const endPoint = cm.getCursor('end')
  cm.replaceSelection(`@@[${selectedText}]()`)
  cm.focus()

  if (startPoint.line !== endPoint.line || startPoint.ch !== endPoint.ch) {
    endPoint.ch += 3
  }
  startPoint.ch += 3
  cm.setSelection(startPoint, endPoint)
}

function drawLatex(editor) {
  const cm = editor.codemirror
  const selectedText = cm.getSelection()
  const startPoint = cm.getCursor('start')
  const endPoint = cm.getCursor('end')
  cm.replaceSelection(`$$${selectedText}$$`)
  cm.focus()

  if (startPoint.line !== endPoint.line || startPoint.ch !== endPoint.ch) {
    endPoint.ch += 2
  }
  startPoint.ch += 2
  cm.setSelection(startPoint, endPoint)
}

function uploadAndInsertFile(uploader, accepts, insertAsImage) {
  if (typeof uploader !== 'function') return SimpleMDE.drawImage

  return function (editor) {
    const fileInput = document.createElement('input')
    fileInput.type = 'file'
    fileInput.accept = accepts.join(', ')
    fileInput.setAttribute('multiple', 'multiple')

    fileInput.onchange = e => {
      const files = e.target.files
      if (e.target.files.length <= 0) return
      Promise.all(Array.from(files)
        .map(file => uploader(file)))
        .then(uploadedFiles => {
          const cm = editor.codemirror
          const startPoint = cm.getCursor('start')
          const uploadedFileStr = uploadedFiles
            .map((file, index) => (insertAsImage
                ? `\n![${files[index].name || ''}](${file.url})`
                : `\n[${files[index].name || ''}](${file.url})`
            ))
            .join('') + '\n'
          cm.replaceSelection(uploadedFileStr)
          cm.focus()

          startPoint.ch = 0
          startPoint.line += uploadedFiles.length + 1
          cm.setCursor(startPoint)
        })
    }
    fileInput.click()
  }
}

export function getButtons(buttonNames, config = {}) {
  const { lng, uploader } = config
  if (!buttonNames) return
  const localButtons = {}
  Object.keys(buttons).forEach(key => {
    const button = buttons[key]
    if (typeof button === 'string') {
      localButtons[key] = button
    } else {
      localButtons[key] = Object.assign({}, button, {
        title: i18n.t(`button:${key}`, { lng })
      })
      if (key === 'upload-image') {
        localButtons[key].action = uploadAndInsertFile(uploader, ['image/*'], true)
      }
      if (key === 'upload-file') {
        localButtons[key].action = uploadAndInsertFile(uploader, [
          // https://github.com/pintia/sparkling-daydream/blob/master/src/main/kotlin/cn/patest/sparklingdaydream/util/NosUtil.kt
          'application/pdf',
          'application/msword', // .doc
          'application/vnd.ms-word', // .doc
          'application/vnd.openxmlformats-officedocument.wordprocessingml.document', // .docx
          'application/vnd.ms-powerpoint', // .ppt
          'application/vnd.openxmlformats-officedocument.presentationml.presentation', // .pptx
          'application/vnd.ms-excel', // .xls
          'application/vnd.ms-spreadsheet',  // .xls
          'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', // .xlsx
        ])
      }
      if (key === 'guide' && config.guide) {
        localButtons[key].action = config.guide
      }
    }
  })
  return buttonNames.map(buttonName => localButtons[buttonName] || buttonName)
}
