const fs = require('fs')
const { expect } = require('chai')
const jsdom = require('jsdom')
global.window = new jsdom.JSDOM('', {
  features: {
    FetchExternalResources: false, // disables resource loading over HTTP / filesystem
    ProcessExternalResources: false // do not execute JS within script blocks
  }
}).window


const ques = require('../lib/ques')

const sample_input = name => fs.readFileSync(`${__dirname}/samples/${name}.md`, { encoding: 'utf8' })
const sample_output = name => fs.readFileSync(`${__dirname}/samples/${name}.html`, { encoding: 'utf8' })

describe('ques', () => {

  it('works for basic markdown', () => {
    expect(ques.parse(sample_input('MarkdownBasic'))).to.equal(sample_output('MarkdownBasic'))
  })

  it('works for basic katex', () => {
    expect(ques.parse(sample_input('katex'))).to.equal(sample_output('katex'))
  })

  it('works for code', () => {
    expect(ques.parse(sample_input('code'))).to.equal(sample_output('code'))
  })

  it('should not truncate spaces after sample input/output', () => {
    expect(ques.parse('```in\n   input   \n```')).to.equal('<div class="ques-view"><pre><code class="lang-in">   input   \n</code></pre>\n</div>\n')
  })


  it('works for true or false (true)', () => {
    expect(ques.parse(sample_input('TrueOrFalse_T'))).to.equal(sample_output('TrueOrFalse_T'))
  })

  it('works for true or false (false)', () => {
    expect(ques.parse(sample_input('TrueOrFalse_F'))).to.equal(sample_output('TrueOrFalse_F'))
  })

  it('works for multiple choice (1)', () => {
    expect(ques.parse(sample_input('MultipleChoice_1'))).to.equal(sample_output('MultipleChoice_1'))
  })

  it('works for multiple choice (M)', () => {
    expect(ques.parse(sample_input('MultipleChoice_M'))).to.equal(sample_output('MultipleChoice_M'))
  })

  it('works for fill in blank', () => {
    expect(ques.parse(sample_input('FillInBlank'))).to.equal(sample_output('FillInBlank'))
  })
})
